<?php
/*
 * API Export controller
 *
 * NOTICE OF LICENSE
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/OSL-3.0
 *
 * DISCLAIMER
 * Do not edit or add to this file if you wish to upgrade Bizuno to newer
 * versions in the future. If you wish to customize Bizuno for your
 * needs please refer to http://www.phreesoft.com for more information.
 *
 * @name       Bizuno ERP
 * @author     Dave Premo, PhreeSoft <support@phreesoft.com>
 * @copyright  2008-2023, PhreeSoft, Inc.
 * @license    http://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * @version    6.x Last Update: 2023-05-08
 * @filesource /controllers/bizuno/export.php
 */

namespace bizuno;

class bizunoExport
{
    public  $moduleID  = 'bizuno';
    public  $user_email= '';      // will be filled in after access granted
    private $updated   = false;
    private $endCatOnly= false;   // if set to true, will only check the final category of the tree

    function __construct() {
        $this->lang = getLang('bizuno'); // needs to be hardcoded as this is extended by extensions
    }

    /**
     *
     */
    public function apiInventory(&$product=[])
    {
        $rID = clean($product['RecordID'], 'integer');
        if (!$rID) { return msgDebug("\nBad ID passed. Needs to be the inventory field id tag name (RecordID)."); }
        // load prices
        setUserCache('security', 'prices_c', 1);
        setUserCache('security', 'j12_mgr', 1);
        msgDebug("\nRead back user cache for prices_c = ".getUserCache('security', 'prices_c'));
        msgDebug("\nRead back user cache for j12_mgr = ".getUserCache('security', 'j12_mgr'));
        bizAutoLoad(BIZBOOKS_ROOT.'controllers/inventory/prices.php', 'inventoryPrices');
        $prices = new inventoryPrices();
        $_GET['rID'] = $rID;
        $pDetails = [];
        $prices->quote($pDetails);
//      msgDebug("\nRetrieved price data in api = ".print_r($pDetails, true));
        $product['Price'] = $pDetails['content']['price'];
        if (!empty($pDetails['content']['regular_price'])){ $product['RegularPrice']= $pDetails['content']['regular_price']; }
        if (!empty($pDetails['content']['sale_price']))   { $product['SalePrice']   = $pDetails['content']['sale_price']; }
        if ( isset($pDetails['content']['sheets']) && sizeof($pDetails['content']['sheets']) > 0) { $product['PriceLevels'] = $pDetails['content']['sheets']; }
        $product['WeightUOM']   = getModuleCache('inventory', 'settings', 'general', 'weight_uom','LB');
        $product['DimensionUOM']= getModuleCache('inventory', 'settings', 'general', 'dim_uom',   'IN');
//        $this->getImage($product);
//        $this->getAccessories($product);
    }

    /**
     * @TODO DEPRECATED
     */
/*    private function getImage(&$product)
    {
        global $io;
        $product['Images'] = [];
        $product['ProductImageData'] = $product['ProductImageDirectory'] = $product['ProductImageFilename'] = '';
        if (!empty($product['Image'])) { // primary image file
            $info = pathinfo($product['Image']);
            $data = $io->fileRead("images/{$product['Image']}");
            $product['ProductImageData']     = $data ? base64_encode($data['data']) : '';
            $product['ProductImageDirectory']= $info['dirname']."/";
            $product['ProductImageFilename'] = $info['basename'];
        }
        if (empty($product['invImages'])) { return; }
        $images = json_decode($product['invImages'], true);
        foreach ($images as $image) { // invImages extension list
            $info = pathinfo($image);
            $data = $io->fileRead("images/$image");
            $product['Images'][] = ['Title'=>$info['basename'], 'Path'=>$info['dirname']."/", 'Filename'=>$info['basename'], 'Data'=>$data ? base64_encode($data['data']) : ''];
        }
    } */

    /**
     * @TODO - DEPRECATED
     */
/*    private function getAccessories(&$product)
    {
        if (isset($product['invAccessory'])) {
            $vals = json_decode($product['invAccessory'], true);
            if (!is_array($vals)) { return; }
            unset($product['invAccessory']);
            foreach ($vals as $rID) {
                $product['invAccessory'][] = dbGetValue(BIZUNO_DB_PREFIX.'inventory', 'sku', "id=$rID");
            }
        }
    } */

    /**
     * @TODO - DEPRECATED
     */
    public function apiSync(&$output)
    {
        $output['syncDelete'] = clean('syncDelete', 'integer', 'get');
        $field  = !empty($output['syncTag']) ? $output['syncTag'] : 'cart_sync';
        $result = dbGetMulti(BIZUNO_DB_PREFIX.'inventory', "`$field`='1' AND inactive='0'");
        foreach ($result as $row) { $output['syncSkus'][] = $row['sku']; }
        msgDebug("\nSending aipSync content = ".print_r($output, true));
    }

    /**
     * @TODO - DEPRECATED
     */
    public function uploadSync($output)
    {
        $missing_skus = [];
        $cartProducts = wpdbGetMulti(PORTAL_DB_PREFIX.'postmeta', "meta_key='_sku'", 'meta_key');
        foreach ($cartProducts as $product_db) {
            if (!in_array($product_db['meta_value'], $output['syncSkus'])) {
                msgDebug("\nMissing product meta: ".print_r($product_db, true));
                $title = wpdbGetValue(PORTAL_DB_PREFIX.'posts', 'post_title', "ID='{$product_db['post_id']}'");
                $missing_skus[] = $title.' (SKU: '.$product_db['meta_value'].')';
                if (!empty($output['syncDelete'])) { wp_delete_post((int)$product_db['post_id']); }
            }
        }
        if (sizeof($missing_skus) > 0) { return msgAdd($this->lang['msg_sku_missing'].'Bizuno:'.'<br>'.implode('<br>', $missing_skus), 'caution'); }
        msgAdd($this->lang['msg_sku_sync_success'], 'success');
    }

    /**
     * @TODO - DEPRECATED
     */
    public function apiConfirm(&$output)
    {
        $date = clean('dateShip', ['format'=>'date', 'default'=>biz_date('Y-m-d')], 'get');
        if (empty($date)) { $date = biz_date('Y-m-d'); }
        $this->confirmShip($output, $date);
        if (empty($output)) { msgAdd('No shipments could be found for this date!', 'success'); }

    }

    /**
     * Looks for product and replaces or creates a new product
     * @param array $orders - data array of the product information
     * @return null - messages are added with status
     */
    public function uploadConfirm($orders=[])
    {
        $order_cnt = 0;
        $order_list= [];
        $prefix    = get_option('bizuno_api_prefix');
        $status    = get_option('bizuno_ship_status');
        $newStatus = $status ? $status : 'wc-completed';
        foreach ($orders as $id => $value) {
            msgDebug("\nWorking with id = $id and value = ".print_r($value, true));
            // strip prefix from order ref (WC3044 => 3044) that is the record number
            if     ($prefix && (strpos($id, $prefix) !== 0 || strpos($id, $prefix) === false)) { continue; }
            elseif ($prefix && strpos($id, $prefix) === 0) { $id = substr($id, strlen($prefix)); }
            $result = wpdbGetValue(PORTAL_DB_PREFIX.'posts', ['post_status','comment_status','comment_count'], "ID=".intval($id)." AND post_type='shop_order'");
            // if not found OR wp-completed continue - no need to duplicate
            if (empty($result) || $result['post_status'] == $newStatus) { msgDebug("\nSkipping id=$id"); continue; }
            // change the status to (wp-completed, closed, bump comment_count)
            $data = ['post_status'=>$newStatus,'comment_status'=>'closed','comment_count' =>$result['comment_count']+1];
            wpdbWrite(PORTAL_DB_PREFIX.'posts', $data, 'update', "ID='$id'");
            // insert into wp_comments per table fields with tracking, ship date etc.
            $comment = [
                'comment_post_ID'     => $id,
                'comment_author'      => 'Bizuno_Interfce',
                'comment_author_email'=> $this->user_email,
                'comment_date'        => wp_date('Y-m-d H:i:s'),
                'comment_date_gmt'    => gmdate('Y-m-d H:i:s'),
                'comment_content'     => $value['Tracking'],
                'comment_agent'       => 'WooCommerce',
                'comment_type'        => 'order_note',
            ];
            wpdbWrite(PORTAL_DB_PREFIX.'comments', $comment);
            $order_cnt++;
            $order_list[] = $id;
        }
        msgAdd(sprintf($this->lang['confirm_success'], sizeof($order_list), sizeof($order_list)>0?" (".implode(', ', $order_list).")":''), 'success');
        msgLog(sprintf($this->lang['ship_confirm_log'], biz_date()));
    }

    /**
     * This method handles orders that have shipped on the specified date
     * @param array $output - working list of orders and shipping information
     * @param string $confirmDate - date to use as the ship date of the order
     */
    private function confirmShip(&$output, $confirmDate=false)
    {
        if (!$confirmDate) { return; }
        if (!dbTableExists(BIZUNO_DB_PREFIX.'extShipping')) {
            return msgAdd("The shipping extension is required for this operation.");
        }
        $result = dbGetMulti(BIZUNO_DB_PREFIX.'extShipping', "ship_date LIKE '$confirmDate%'");
        if (sizeof($result) == 0) { return msgAdd($this->lang['msg_no_shipments_found'], 'caution'); }
        // process the ship list
        foreach ($result as $value) {
            $invoice_num = strpos($value['ref_id'], '-') ? substr($value['ref_id'], 0, strpos($value['ref_id'], '-')) : $value['ref_id'];
            $value['order_num'] = dbGetValue(BIZUNO_DB_PREFIX.'journal_main', 'purch_order_id', "invoice_num='$invoice_num'");
            $order_num = $value['order_num'] ? $value['order_num'] : $invoice_num;
            $method_text = viewProcess($value['method_code'], 'shipInfo');
            if (isset($output[$order_num]['tracking_id'])) {
                $output[$order_num]['Tracking'] .= ', '.$value['tracking_id'];
            } else {
                $output[$order_num]['Tracking'] = 'Your order shipped '.viewDate($confirmDate).' via '.$method_text.', tracking number(s): '.$value['tracking_id'];
            }
            if (isset($output[$order_num]['notes'])) {
                $output[$order_num]['Notes'] .= '. '.$value['notes'];
            } else {
                if ($value['notes']) { $output[$order_num]['Notes'] = lang('notes').': '.$value['notes']; }
            }
            $output[$order_num]['Method']   = $method_text;
            $output[$order_num]['TrackID']  = $value['tracking_id'];
            $output[$order_num]['Status']   = 'orderShipped';
            $output[$order_num]['ShipDate'] = $confirmDate;
        }
    }

    /**
     * Looks for product and replaces or creates a new product
     * @param array $product - data array of the product information
     * @return null - messages are added with status
     */
    public function uploadProduct($product=[])
    {
        set_time_limit(240); // if uploading large images for firs time, it can take a while...
        if (empty($product)) { msgAdd("Bad product string received, check the error log.", 'trap'); }
        msgDebug("\nReached uploadProduct with size of product array = ".sizeof($product));
        if (empty($product['SKU'])) { return msgAdd("Cannot find product SKU."); }
        // call the REST API to complete the upload

    }

    /**
     * Takes package information in and returns the Bizuno enabled shipping rates and services
     * @param array $layout - Structure coming in ($layout['pkg'] is where the information is)
     */
    public function shippingRates(&$layout=[])
    {
        if ( !\is_plugin_active( 'bizuno-pro/bizuno-pro.php' ) ) { return; }
        $pkg = [
            'ship' => [
                'address1'   => $layout['pkg']['destination']['address'],
                'address2'   => $layout['pkg']['destination']['address_1'],
                'city'       => $layout['pkg']['destination']['city'],
                'state'      => $layout['pkg']['destination']['state'],
                'postal_code'=> $layout['pkg']['destination']['postcode'],
                'country'    => clean($layout['pkg']['destination']['country'], ['format'=>'country','option'=>'ISO3'])],
            'settings' => [
                'ship_date'  => date('Y-m-d'),
                'order_total'=> $layout['pkg']['cart_subtotal'],
                'weight'     => !empty($layout['pkg']['destination']['totalWeight']) ? $layout['pkg']['destination']['totalWeight'] : 1,
//              'length'     => clean('length',     'float',  'post'),
//              'width'      => clean('width',      'float',  'post'),
//              'height'     => clean('height',     'float',  'post'),
                'num_boxes'  => 1, // Assume 1 for now as dims may not be entered
                'ltl_class'  => 60, // clean('ltl_class',  'text',   'post'),
                'residential'=> 1,  // clean('residential','boolean','post'),
                'verify_add' => true]];
        $pkg['ship']['country2'] = $layout['pkg']['destination']['country'];
        $pkg['ship']['country3'] = $pkg['ship']['country'];
        bizAutoLoad(BIZBOOKS_EXT.'controllers/proLgstc/ship.php', 'proLgstcShip');
        $quote = new proLgstcShip();
        $quote->getRateAddress($pkg, 'bill');
        $quote->getRateAddress($pkg, 'origin');
        $layout['rates'] = $quote->rateAPI($pkg);
    }

    /**
     *
     * @param unknown $value
     * @param unknown $unit_original m,cm,mm,in,ft
     * @param unknown $unit_return m,cm,mm,in,yd
     */
/*    private function convertLength($value, $unit_original, $unit_return) {
        if($unit_original == $unit_return) { return $value; }
        switch ($unit_original) {
            case 'm':
                switch($unit_return) {
                    case 'cm': return round($value/100, 2);
                    case 'mm': return round($value/1000, 2);
                    case 'in': return round($value*39.370, 2);
                    case 'yd': return round($value/1.0936, 2);
                }
            case 'cm':
                switch($unit_return) {
                    case  'm': return round($value*100, 2);
                    case 'mm': return round($value/10, 2);
                    case 'in': return round($value*0.39370, 2);
                    case 'yd': return round($value/109.36, 2);
                }
            case 'mm':
                switch($unit_return) {
                    case  'm': return round($value*1000, 2);
                    case 'cm': return round($value*10, 2);
                    case 'in': return round($value*0.039370, 2);
                    case 'yd': return round($value*0.039370*3*12, 2);
                }
            case 'in':
                switch($unit_return) {
                    case  'm': return round($value/39.370, 2);
                    case 'cm': return round($value/0.39370, 2);
                    case 'mm': return round($value/0.039370, 2);
                    case 'yd': return round($value*3*12, 2);
                }
            case 'ft':
                switch($unit_return) {
                    case  'm': return round($value/3.2808, 2);
                    case 'cm': return round($value/0.032808, 2);
                    case 'mm': return round($value/0.0032808, 2);
                    case 'yd': return round($value*3, 2);
                    case 'in': return round($value/12, 2);
                }
            default:
                msgAdd("length conversion Error","warning");
                return $value;
        }
    } */

    /**
     *
     * @param unknown $value
     * @param unknown $unit_original kg,lb
     * @param unknown $unit_return kg,g,lb,oz
     * @return void|unknown|string|number
     */
/*    private function convertWeight($value, $unit_original, $unit_return) {
        msgDebug("\nEntering convertWeight with value = $value, unit_original = $unit_original and unit_return = $unit_return");
        switch ($unit_original) {
            case 'kg':
            case 'kgs':
                if (in_array($unit_original, ['kg', 'kgs'])) { return value; }
                switch($unit_return) {
                    case  'g': return round($value/1000, 3);
                    case 'lb':
                    case 'lbs':return round($value*2.2046, 2);
                    case 'oz': return round($value*35.274, 2);
                }
            case 'lb':
            case 'lbs':
                if (in_array($unit_original, ['lb', 'lbs'])) { return $value; }
                switch($unit_return) {
                    case 'kg':
                    case 'kgs':return round($value/2.2046, 3);
                    case  'g': return round($value/0.0022046, 3);
                    case 'oz': return round($value*16, 2);
                }
            default:
                msgAdd("Weight Conversion Error, received original unit = $unit_original", 'warning');
                return $value;
        }
    } */

    /**
     * Install common fields into inventory db table shared amongst all the interfaces
     */
    protected function installStoreFields()
    {
        $id = validateTab($module_id='inventory', 'inventory', lang('estore'), 80);
        if (!dbFieldExists(BIZUNO_DB_PREFIX."inventory", 'msrp'))             { dbGetResult("ALTER TABLE ".BIZUNO_DB_PREFIX."inventory ADD msrp DOUBLE NOT NULL DEFAULT '0' COMMENT 'label:Mfg Suggested Retail Price;tag:MSRPrice;tab:$id;order:42'"); }
        // All of these fields have been moved to attributes or removed. Used for customization only.
//      if (!dbFieldExists(BIZUNO_DB_PREFIX."inventory", 'description_long')) { dbGetResult("ALTER TABLE ".BIZUNO_DB_PREFIX."inventory ADD description_long TEXT COMMENT 'type:textarea;label:Long Description;tag:DescriptionLong;tab:$id;order:10'"); }
//      if (!dbFieldExists(BIZUNO_DB_PREFIX."inventory", 'manufacturer'))     { dbGetResult("ALTER TABLE ".BIZUNO_DB_PREFIX."inventory ADD manufacturer VARCHAR(24) NOT NULL DEFAULT '' COMMENT 'label:Manufacturer;tag:Manufacturer;tab:$id;order:40'"); }
//      if (!dbFieldExists(BIZUNO_DB_PREFIX."inventory", 'model'))            { dbGetResult("ALTER TABLE ".BIZUNO_DB_PREFIX."inventory ADD model VARCHAR(24) NOT NULL DEFAULT '' COMMENT 'label:Model;tag:Model;tab:$id;order:41'"); }
//      if (!dbFieldExists(BIZUNO_DB_PREFIX."inventory", 'meta_keywords'))    { dbGetResult("ALTER TABLE ".BIZUNO_DB_PREFIX."inventory ADD meta_keywords VARCHAR(255) NOT NULL DEFAULT '' COMMENT 'label:Meta Keywords;tag:MetaKeywords;tab:$id;o rder:90;group:General'"); }
//      if (!dbFieldExists(BIZUNO_DB_PREFIX."inventory", 'meta_description')) { dbGetResult("ALTER TABLE ".BIZUNO_DB_PREFIX."inventory ADD meta_description VARCHAR(255) NOT NULL DEFAULT '' COMMENT 'label:Meta Description;tag:MetaDescription;tab:$id;order:91;group:General'"); }
        bizAutoLoad(BIZBOOKS_ROOT.'controllers/inventory/admin.php', 'inventoryAdmin');
        $inv = new inventoryAdmin();
        $inv->installPhysicalFields();
    }
}
