<?php
/*
 * Table structure for Bizuno core
 *
 * NOTICE OF LICENSE
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/OSL-3.0
 *
 * DISCLAIMER
 * Do not edit or add to this file if you wish to upgrade Bizuno to newer
 * versions in the future. If you wish to customize Bizuno for your
 * needs please refer to http://www.phreesoft.com for more information.
 *
 * @name       Bizuno ERP
 * @author     Dave Premo, PhreeSoft <support@phreesoft.com>
 * @copyright  2008-2023, PhreeSoft, Inc.
 * @license    http://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * @version    6.x Last Update: 2022-06-25
 * @filesource /controllers/bizuno/install/tables.php
 */
namespace bizuno;

$iso = getDefaultCurrency();
$tables = [
    'address_book' => ['module' => 'contacts',
        'fields'=> [
            'address_id'  => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;format:integer;tag:RecordID;order:1',
                'import'=>false, 'export'=>false,'required'=>true,'desc'=>"Address Book table ID field"],
            'ref_id'      => ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ReferenceID;order:2',
                'import'=>false, 'export'=>false,'required'=>true,'desc'=>"ID field in contacts table to link address record to."],
            'type'        => ['format'=>'CHAR(1)',    'attr'=>"NOT NULL DEFAULT 'c'",   'comment'=>'type:hidden;tag:AddressType;order:3',
                'import'=>false, 'export'=>false,'required'=>true,'desc'=>"Address book type, choices are m (Main), b (Billing), or s (Shipping)"],
            'primary_name'=> ['format'=>'VARCHAR(48)','attr'=>"DEFAULT NULL",           'comment'=>'tag:PrimaryName;order:10',
                'import'=>true, 'export'=>true,'required'=>true,'desc'=>"Primary or Business Name as it appears in correspondence"],
            'contact'     => ['format'=>'VARCHAR(48)','attr'=>"DEFAULT NULL",           'comment'=>'tag:Contact;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Contact or Attention, typically a person or department"],
            'address1'    => ['format'=>'VARCHAR(48)','attr'=>"DEFAULT NULL",           'comment'=>'tag:Address1;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Address field 1"],
            'address2'    => ['format'=>'VARCHAR(48)','attr'=>"DEFAULT NULL",           'comment'=>'tag:Address2;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Address field 2"],
            'city'        => ['format'=>'VARCHAR(24)','attr'=>"DEFAULT NULL",           'comment'=>'tag:City;order:50',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"City or town"],
            'state'       => ['format'=>'VARCHAR(24)','attr'=>"DEFAULT NULL",           'comment'=>'tag:State;order:60',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"State of province"],
            'postal_code' => ['format'=>'VARCHAR(10)','attr'=>"DEFAULT NULL",           'comment'=>'tag:PostalCode;order:70',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Postal or Zip code"],
            'country'     => ['format'=>'CHAR(3)',    'attr'=>"NOT NULL DEFAULT 'USA'", 'comment'=>'type:country;tag:CountryISO3;order:80',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Three character ISO country code, i.e. USA, JPN, CAN"],
            'telephone1'  => ['format'=>'VARCHAR(20)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Telephone1;order:10',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Telephone number 1 (Primary)"],
            'email'       => ['format'=>'VARCHAR(64)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Email;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Email address 1 (Primary)"],
            'telephone2'  => ['format'=>'VARCHAR(20)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Telephone2;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Telephone number 2 (Secondary)"],
            'email2'      => ['format'=>'VARCHAR(64)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Email2;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Email address 2 (Secondary)"],
            'telephone3'  => ['format'=>'VARCHAR(20)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Telephone3;order:50',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Telephone Number 3 (Fax)"],
            'email3'      => ['format'=>'VARCHAR(64)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Email3;order:60',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Email address 3 (Sales)"],
            'telephone4'  => ['format'=>'VARCHAR(20)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Telephone4;order:70',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Telephone Number 4 (Mobile)"],
            'email4'      => ['format'=>'VARCHAR(64)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Email4;order:80',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Email address 4 (Accounting)"],
            'website'     => ['format'=>'VARCHAR(48)','attr'=>"NULL DEFAULT ''",        'comment'=>'tag:Website;order:90',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Website address"],
            'notes'       => ['format'=>'TEXT',       'attr'=>"",                       'comment'=>'tag:Notes;order:95',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Address notes. Text field - 64,000 characters max."]],
        'keys' => 'PRIMARY KEY (address_id), KEY customer_id (ref_id, type)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'audit_log' => ['module' => 'bizuno',
        'fields' => [
            'id'          => ['format'=>'INT(15)',     'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'module_id'   => ['format'=>'VARCHAR(24)', 'attr'=>"DEFAULT ''",             'comment'=>'type:hidden;tag:ModuleID;order:2'],
            'ip_address'  => ['format'=>'VARCHAR(15)', 'attr'=>"DEFAULT ''",             'comment'=>'type:hidden;tag:IPAddress;order:3'],
            'user_id'     => ['format'=>'INT(11)',     'attr'=>"DEFAULT '0'",            'comment'=>'tag:UserID;order:10'],
            'date'        => ['format'=>'TIMESTAMP',   'attr'=>"NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP",'comment'=>'type:date;tag:LogDate;order:20'],
            'stats'       => ['format'=>'VARCHAR(32)', 'attr'=>"DEFAULT ''",             'comment'=>'type:hidden;tag:Statistics;order:30'],
            'log_entry'   => ['format'=>'VARCHAR(255)','attr'=>"DEFAULT NULL",           'comment'=>'tag:LogEntry;order:50']],
        'keys' => 'PRIMARY KEY (id), KEY (user_id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'configuration' => ['module' => 'bizuno',
        'fields' => [
            'config_key'  => ['format'=>'VARCHAR(64)','attr'=>"NOT NULL DEFAULT ''",'comment'=>'type:hidden;tag:ConfigKey;order:10'],
            'config_value'=> ['format'=>'MEDIUMTEXT', 'attr'=>"",                   'comment'=>'type:hidden;tag:ConfigValue;order:20']],
        'keys' => 'PRIMARY KEY (config_key)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'contacts' => ['module' => 'contacts',
        'fields'=> [
            'id'            => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=> 'type:hidden;tag:RecordID;order:1',
                'import'=>false,'export'=>false,'required'=>true, 'desc'=>"Contact table ID field"],
            'type'          => ['format'=>'CHAR(1)',      'attr'=>"NOT NULL DEFAULT 'c'",   'comment'=> 'type:hidden;tag:Type;order:2',
                'import'=>true, 'export'=>true, 'required'=>true,'desc'=>"Contact type, choices are: c (Customer), v (Vendor), e (Employee), i (CRM Contact), b (Store or Branch). See documentation for more options"],
            'attach'        => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:hidden;tag:Attachment;order:3',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Attachments Present, valid values are 0 - Not Present, or 1 - Present"],
            'short_name'    => ['format'=>'VARCHAR(32)',  'attr'=>"NOT NULL DEFAULT ''",    'comment'=> 'tag:ContactID;order:10',
                'import'=>true, 'export'=>true, 'required'=>true, 'desc'=>"Contact ID, short name to identify contact, max 32 characters"],
            'inactive'      => ['format'=>"CHAR(1)",      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=> 'type:select;tag:Status;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[default: 0] Contact status, 0 - Active, 1 - Active, 2 - Locked"],
            'store_id'      => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=> 'type:hidden;tag:StoreID;order:25',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Store ID, contact record number of store associated to this contact"],
            'tax_exempt'    => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:checkbox;tag:TaxExempt;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Contact is Tax Exempt, valid values are 0 - No, or 1 - Yes"],
            'tax_rate_id'   => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=> 'type:tax;format:integer;tag:TaxRateID;order:32',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Default Sales tax, this is the ID from the tax_rates table"],
            'rep_id'        => ['format'=>'VARCHAR(16)',  'attr'=>"DEFAULT NULL",           'comment'=> 'type:select;tag:DeptRepID;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Department Rep ID, contact record ID of the user assigned to this account"],
            'price_sheet'   => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=> 'type:select;tag:PriceSheetID;order:50',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Default price sheet for the user, this is a record ID from the inventory prices table"],
            'terms'         => ['format'=>'VARCHAR(32)',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=> 'type:hidden;tag:Terms;order:60',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[default: 0] Payment terms, this is an encoded value and should generally be not included in the import feed"],
            'contact_first' => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT NULL",           'comment'=> 'tag:FirstName;order:70',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Contact First Name"],
            'contact_last'  => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT NULL",           'comment'=> 'tag:LastName;order:80',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Contact Last Name"],
            'flex_field_1'  => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT NULL",           'comment'=> 'tag:Title;order:85',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Contact title for Customers, Vendors, middle name for Employees"],
            'account_number'=> ['format'=>'VARCHAR(16)',  'attr'=>"DEFAULT NULL",           'comment'=> 'tag:AccountNumber;order:10',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Account Number for vendors this would be your account number assigned by the vendor"],
            'gov_id_number' => ['format'=>'VARCHAR(16)',  'attr'=>"DEFAULT NULL",           'comment'=> 'tag:GovID;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Government ID Number, i.e. SSN for US citizens"],
            'gl_account'    => ['format'=>'VARCHAR(15)',  'attr'=>"DEFAULT NULL",           'comment'=> 'type:ledger;tag:DefaultGLAccount;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Default GL Account for purchases/sales"],
            'first_date'    => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=> 'type:date;tag:DateCreated;order:70',
                'import'=>true,'export'=>true,'required'=>false,'desc'=>"The database record creation date for this contact"],
            'last_update'   => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=> 'type:date;tag:DateLastEntry;order:75',
                'import'=>true,'export'=>true,'required'=>false,'desc'=>"The last update for this contact."],
            'last_date_1'   => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=> 'type:date;tag:AltDate1;order:80',
                'import'=>true,'export'=>true,'required'=>false,'desc'=>"Last Date 1, uses vary depending on contact type"],
            'last_date_2'   => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=> 'type:date;tag:AltDate2;order:85',
                'import'=>true,'export'=>true,'required'=>false,'desc'=>"Last Date 2, uses vary depending on contact type"]],
        'keys' => 'PRIMARY KEY (id), KEY type (type), KEY short_name (short_name)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci',
        'extra_tabs' => true],
    'contacts_log' => ['module' => 'contacts',
        'fields'=> [
            'id'        => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'contact_id'=> ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ContactID;order:2'],
            'log_date'  => ['format'=>'DATETIME',   'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:LogDate;order:10'],
            'entered_by'=> ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:UserID;order:20'],
            'action'    => ['format'=>'VARCHAR(32)','attr'=>"DEFAULT NULL",           'comment'=>'tag:Action;order:30'],
            'notes'     => ['format'=>'TEXT',       'attr'=>"",                       'comment'=>'tag:Notes;order:40']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'current_status' => ['module' => 'bizuno',
        'fields' => [
            'id'              => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",    'comment'=>'type:hidden;tag:RecordID;order:1'],
            'next_cust_id_num'=> ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'C10000'",  'comment'=>'tag:ContactC;order:2'],
            'next_vend_id_num'=> ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'V10000'",  'comment'=>'tag:ContactV;order:3'],
            'next_ref_j2'     => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'GL000001'",'comment'=>'tag:Journal2;order:4'],
            'next_ref_j3'     => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'RFQ1000'", 'comment'=>'tag:Journal3;order:5'],
            'next_ref_j4'     => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'PO5000'",  'comment'=>'tag:Journal4;order:6'],
            'next_ref_j7'     => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'VCM1000'", 'comment'=>'tag:Journal5;order:7'],
            'next_ref_j9'     => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'QU1000'",  'comment'=>'tag:Journal9;order:8'],
            'next_ref_j10'    => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'SO0001'",  'comment'=>'tag:Journal10;order:9'],
            'next_ref_j12'    => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT '200000'",  'comment'=>'tag:Journal12;order:10'],
            'next_ref_j13'    => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'CM1000'",  'comment'=>'tag:Journal13;order:11'],
            'next_ref_j18'    => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT 'DP00001'", 'comment'=>'tag:Journal18;order:12'],
            'next_ref_j20'    => ['format'=>'VARCHAR(16)','attr'=>"NOT NULL DEFAULT '100'",     'comment'=>'tag:Journal20;order:13'],
            'next_tab_id'     => ['format'=>'int(11)',    'attr'=>"NOT NULL DEFAULT '1'",       'comment'=>'type:hidden;tag:NextTabID;order:14']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'data_security' => ['module' => 'bizuno',
        'fields' => [
            'id'       => ['format'=>'INT(11)',     'attr'=>"NOT NULL AUTO_INCREMENT",      'comment'=>'type:hidden;tag:RecordID;order:1'],
            'module'   => ['format'=>'VARCHAR(32)', 'attr'=>"NOT NULL DEFAULT ''",          'comment'=>'type:hidden;tag:ModuleID;order:2'],
            'ref_1'    => ['format'=>'INT(11)',     'attr'=>"NOT NULL DEFAULT '0'",         'comment'=>'type:hidden;tag:Reference1;order:3'],
            'enc_value'=> ['format'=>'VARCHAR(255)','attr'=>"NOT NULL DEFAULT ''",          'comment'=>'type:hidden;tag:EncryptedValue;order:4'],
            'hint'     => ['format'=>'VARCHAR(255)','attr'=>"NOT NULL DEFAULT ''",          'comment'=>'tag:Hint;order:10'],
            'exp_date' => ['format'=>'DATE',        'attr'=>"NOT NULL DEFAULT '2035-12-31'",'comment'=>'type:date;tag:ExpirationDate;order:20']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'inventory' => ['module' => 'inventory',
        'fields' => [
            'id'                  => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT", 'comment'=>'type:hidden;tag:RecordID;order:1',
                'import'=>false,'export'=>false,'required'=>true,'desc'=>"Inventory table record ID"],
            'attach'              => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:hidden;tag:Attachment;order:4',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Attachments Present, valid values are 0 - Not Present, or 1 - Present"],
            'sku'                 => ['format'=>'VARCHAR(24)',  'attr'=>"NOT NULL DEFAULT ''",     'comment'=>'tag:SKU;order:10',
                'import'=>true, 'export'=>true,'required'=>true,'desc'=>"Item SKU, must be unique, duplicates will override the current values."],
            'description_short'   => ['format'=>'VARCHAR(64)',  'attr'=>"DEFAULT NULL",            'comment'=>'tag:Description;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Required for inital import, optional thereafter"],
            'inactive'            => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:checkbox;tag:Inactive;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Inactive flag, valid values are 0 - Active, or 1 - Inactive"],
            'item_weight'         => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:Weight;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Item weight (in default units)"],
            'upc_code'            => ['format'=>'VARCHAR(13)',  'attr'=>"DEFAULT NULL",            'comment'=>'tag:UPCCode;order:50',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"UPC, EAN, JAN, or ISBN. Universal standard product code."],
            'lead_time'           => ['format'=>'INT(3)',       'attr'=>"NOT NULL DEFAULT '1'",    'comment'=>'tag:StockLeadTime;order:60',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Lead time (in days) to receive product from the vendor when an order is placed."],
            'qty_stock'           => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:QtyStock;order:10',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Quantity in Stock (calculated by the system)"],
            'qty_min'             => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:StockMinimum;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Minimum stock level to trigger re-order."],
            'qty_restock'         => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:StockReorder;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Restock quantity. Used to set quantites to reorder from vendor."],
            'qty_po'              => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:QtyPurchase;order:40',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Quantity on Purchse Order (calculated by the system)"],
            'qty_so'              => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:QtyOrder;order:50',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Quantity on Sales Order (calculated by the system)"],
            'qty_alloc'           => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'tag:QtyAllocation;order:60',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Quantity on Allocation (calculated by the system)"],
            'store_id'            => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:hidden;tag:StoreID;order:52',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Preferred Store ID. ID value is database record id from the contacts table, default -1 (all)."],
            'description_sales'   => ['format'=>'VARCHAR(255)', 'attr'=>"DEFAULT NULL",            'comment'=>'tag:DescriptionSales;order:10',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Detailed description used for sales"],
            'tax_rate_id_c'       => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:select;tag:TaxRateIDCustomer;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[Default: 0] Tax Rate ID for customers from the db table"],
            'price_sheet_c'       => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:select;tag:DefPriceSheetIDSales;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[Default: 0] Default price sheet for customers"],
            'full_price'          => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:currency;tag:FullPrice;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Retail or List price for the item"],
            'sale_price'          => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:currency;tag:SalePrice;order:42',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Sale price for the item"],
            'description_purchase'=> ['format'=>'VARCHAR(255)', 'attr'=>"DEFAULT NULL",            'comment'=>'tag:DescriptionPurchase;order:10',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Detailed decription to use for purchases"],
            'tax_rate_id_v'       => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:select;tag:TaxRateIDVendor;order:20',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[Default: 0] Tax Rate ID for vendors from the db table"],
            'price_sheet_v'       => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:select;tag:DefPriceSheetIDPurchase;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[Default: 0] Default price sheet for vendors"],
            'item_cost'           => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:currency;tag:Cost;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Item purchase cost"],
            'vendor_id'           => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:select;tag:StockVendor;order:50',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Preferred vendor ID. ID value is database record id from the contacts table."],
            'inventory_type'      => ['format'=>'CHAR(2)',      'attr'=>"NOT NULL DEFAULT 'si'",   'comment'=>'type:select;tag:Type;order:10',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[Default: si, Stock Item] See help files for inventory type codes. Once SKU has a journal entry against it, the type cannot be changed"],
            'serialize'           => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",    'comment'=>'type:checkbox;tag:Serialized;order:20',
                'import'=>false,'export'=>false,'required'=>false,'desc'=>"Serialized Item"],
            'cost_method'         => ['format'=>"ENUM('a','f','l')",'attr'=>"NOT NULL DEFAULT 'f'",'comment'=>'type:select;tag:CostMethod;order:30',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"[Default: f] Cost Method, choices are f (FIFO), l (LIFO), or a (Average)"],
            'gl_sales'            => ['format'=>'VARCHAR(15)',  'attr'=>"DEFAULT NULL",            'comment'=>'type:ledger;tag:GLAccountSales;order:40',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"General Ledger Sales account ID. If left off, the default will be used."],
            'gl_inv'              => ['format'=>'VARCHAR(15)',  'attr'=>"DEFAULT NULL",            'comment'=>'type:ledger;tag:GLAccountInventory;order:50',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"General Ledger Inventory account ID. If left off, the default will be used."],
            'gl_cogs'             => ['format'=>'VARCHAR(15)',  'attr'=>"DEFAULT NULL",            'comment'=>'type:ledger;tag:GLAccountCOGS;order:60',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"General Ledger Cost of Good Sold account ID. If left off, the default will be used."],
            'image_with_path'     => ['format'=>'VARCHAR(255)', 'attr'=>"DEFAULT NULL",            'comment'=>'type:hidden;tag:Image;order:10',
                'import'=>true, 'export'=>true,'required'=>false,'desc'=>"Item image path from your default images folder. i.e. /path/to/image/image.png"],
            'creation_date'       => ['format'=>'DATETIME',     'attr'=>"DEFAULT NULL",            'comment'=>'type:date;tag:DateCreated;order:10',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Item record creation date, format YYY-MM-DD"],
            'last_update'         => ['format'=>'DATETIME',     'attr'=>"DEFAULT NULL",            'comment'=>'type:date;tag:DateLastUpdate;order:20',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Last update. Format YYYY-MM-DD"],
            'last_journal_date'   => ['format'=>'DATETIME',     'attr'=>"DEFAULT NULL",            'comment'=>'type:date;tag:DateLastJournal;order:30',
                'import'=>false,'export'=>true,'required'=>false,'desc'=>"Last date the item was used in a journal entry, format YYY-MM-DD"]],
        'keys' => 'PRIMARY KEY (id), KEY sku (sku)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci',
        'extra_tabs' => true], // allow extra tabs to be added to table
    'inventory_assy_list' => ['module' => 'inventory',
        'fields' => [
            'id'         => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'ref_id'     => ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ReferenceID;order:2'],
            'sku'        => ['format'=>'VARCHAR(24)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'tag:SKU;order:10'],
            'description'=> ['format'=>'VARCHAR(64)','attr'=>"DEFAULT NULL",           'comment'=>'tag:Description;order:20'],
            'qty'        => ['format'=>'FLOAT',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:Quantity;order:30']],
        'keys' => 'PRIMARY KEY (id), KEY ref_id (ref_id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'inventory_history' => ['module' => 'inventory',
        'fields' => [
            'id'        => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'ref_id'    => ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ReferenceID;order:2'],
            'journal_id'=> ['format'=>'INT(2)',     'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:JournalID;order:3'],
            'store_id'  => ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:StoreID;order:4'],
            'sku'       => ['format'=>'VARCHAR(24)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'tag:SKU;order:10'],
            'qty'       => ['format'=>'FLOAT',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:Quantity;order:20'],
            'trans_code'=> ['format'=>'VARCHAR(64)','attr'=>"DEFAULT NULL",           'comment'=>'tag:SerialNumber;order:30'],
            'remaining' => ['format'=>'FLOAT',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:Remaining;order:40'],
            'unit_cost' => ['format'=>'DOUBLE',     'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:UnitCost;order:50'],
            'avg_cost'  => ['format'=>'DOUBLE',     'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:AverageCost;order:60'],
            'post_date' => ['format'=>'DATETIME',   'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:PostDate;order:70']],
        'keys' => 'PRIMARY KEY (id), KEY sku (sku), KEY ref_id (ref_id), KEY remaining (remaining), INDEX (store_id), INDEX (journal_id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'inventory_prices' => ['module' => 'inventory',
        'fields'=> [
            'id'          => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'method'      => ['format'=>'VARCHAR(16)',  'attr'=>"DEFAULT ''",             'comment'=>'tag:PriceMethod;order:10'],
            'ref_id'      => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:ReferenceSheetID;order:20'],
            'contact_type'=> ['format'=>"CHAR(1)",      'attr'=>"DEFAULT 'c'",            'comment'=>'tag:Type;order:30'],
            'contact_id'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ContactID;order:40'],
            'inventory_id'=> ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:InventoryID;order:50'],
            'currency'    => ['format'=>'CHAR(3)',      'attr'=>"NOT NULL DEFAULT '$iso'",'comment'=>'type:hidden;tag:CurrencyISO;order:60'],
            'inactive'    => ['format'=>"ENUM('0','1')",'attr'=>"DEFAULT '0'",            'comment'=>'tag:Inactive;order:70'],
            'settings'    => ['format'=>"TEXT",         'attr'=>"",                       'comment'=>'type:json;tag:Settings;order:80']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'journal_cogs_owed' => ['module' => 'phreebooks',
        'fields' => [
            'id'             => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'journal_main_id'=> ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:JournalRecordID;order:2'],
            'sku'            => ['format'=>'VARCHAR(24)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'tag:SKU;order:10'],
            'qty'            => ['format'=>'FLOAT',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:Quantity;order:20'],
            'store_id'       => ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:StoreID;order:30'],
            'post_date'      => ['format'=>'DATE',       'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:PostDate;order:40']],
        'keys' => 'PRIMARY KEY (id), KEY sku (sku), INDEX (store_id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'journal_cogs_usage' => ['module' => 'phreebooks',
        'fields' => [
            'id'                  => ['format'=>'INT(11)','attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'journal_main_id'     => ['format'=>'INT(11)','attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:JournalRecordID;order:2'],
            'inventory_history_id'=> ['format'=>'INT(11)','attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:HistoryRecordID;order:3'],
            'qty'                 => ['format'=>'FLOAT',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:Quantity;order:4']],
        'keys' => 'PRIMARY KEY (id), INDEX (journal_main_id, inventory_history_id)',
        'attr' => 'ENGINE=innodb DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'journal_history' => ['module' => 'phreebooks',
        'fields'=> [
            'id'               => ['format'=>'INT(11)', 'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'period'           => ['format'=>'INT(11)', 'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:Period;order:10'],
            'gl_account'       => ['format'=>'CHAR(15)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'type:ledger;tag:GLAccount;order:20'],
            'gl_type'          => ['format'=>'INT(2)',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:GLAccountType;order:30'],
            'beginning_balance'=> ['format'=>'DOUBLE',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:BeginningBalance;order:40'],
            'debit_amount'     => ['format'=>'DOUBLE',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:DebitAmount;order:50'],
            'credit_amount'    => ['format'=>'DOUBLE',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:CreditAmount;order:60'],
            'budget'           => ['format'=>'DOUBLE',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:BudgetAmount;order:70'],
            'stmt_balance'     => ['format'=>'DOUBLE',  'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:date;tag:StatementBalance;order:80'],
            'last_update'      => ['format'=>'DATE',    'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:LastUpdate;order:90']],
        'keys' => 'PRIMARY KEY (id), KEY period (period), KEY gl_account (gl_account)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'journal_item' => ['module' => 'phreebooks',
        'fields'=> [
            'id'           => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'ref_id'       => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ReferenceID;order:2'],
            'item_ref_id'  => ['format'=>'INT(11)',      'attr'=>"DEFAULT NULL",           'comment'=>'type:hidden;tag:SoPoItemRefID;order:3'],
            'gl_type'      => ['format'=>'CHAR(3)',      'attr'=>"NOT NULL DEFAULT ''",    'comment'=>'type:hidden;tag:GLType;order:4'],
            'reconciled'   => ['format'=>'INT(4)',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:Reconciled;order:5'],
            'serialize'    => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:Serialize;order:6'],
            'item_cnt'     => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:LineNumber;order:10'],
            'sku'          => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:SKU;order:20'],
            'qty'          => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:Quantity;order:30'],
            'description'  => ['format'=>'VARCHAR(255)', 'attr'=>"DEFAULT NULL",           'comment'=>'tag:Description;order:40'],
            'debit_amount' => ['format'=>'DOUBLE',       'attr'=>"DEFAULT '0'",            'comment'=>'type:currency;tag:DebitAmount;order:50'],
            'credit_amount'=> ['format'=>'DOUBLE',       'attr'=>"DEFAULT '0'",            'comment'=>'type:currency;tag:CreditAmount;order:60'],
            'full_price'   => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:PriceFull;order:65'],
            'gl_account'   => ['format'=>'VARCHAR(15)',  'attr'=>"DEFAULT NULL",           'comment'=>'type:select;tag:GLAccount;order:70'],
            'tax_rate_id'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:select;tag:TaxRateID;order:75'],
            'trans_code'   => ['format'=>'VARCHAR(64)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:TransactionID;order:80'],
            'post_date'    => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:PostDate;order:85'],
            'date_1'       => ['format'=>'DATETIME',     'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:ItemDate1;order:90']],
        'keys' => 'PRIMARY KEY (id), KEY ref_id (ref_id), KEY sku (sku), KEY item_ref_id (item_ref_id), KEY reconciled (reconciled), KEY gl_account (gl_account)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'journal_main' => ['module' => 'phreebooks',
        'fields'=> [
            'id'            => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'period'        => ['format'=>'INT(2)',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:Period;order:5'],
            'journal_id'    => ['format'=>'INT(2)',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:JournalID;order:10'],
            'store_id'      => ['format'=>'INT(11)',      'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:StoreID;order:25'],
            'description'   => ['format'=>'VARCHAR(128)', 'attr'=>"DEFAULT NULL",           'comment'=>'tag:Description;order:30'],
            'discount'      => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:TotalDiscount;order:55'],
            'sales_tax'     => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:TotalTax;order:60'],
            'freight'       => ['format'=>'DOUBLE',       'attr'=>"DEFAULT '0'",            'comment'=>'type:currency;tag:TotalShipping;order:200'],
            'total_amount'  => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:currency;tag:TotalAmount;order:65'],
            'purch_order_id'=> ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:PurchaseOrderID;order:10'],
            'invoice_num'   => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ReferenceNumber;order:20'],
            'closed'        => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:checkbox;tag:Closed;order:30'],
            'waiting'       => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:checkbox;tag:Waiting;order:40'],
            'post_date'     => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:PostDate;order:50'],
            'terminal_date' => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:TerminalDate;order:60'],
            'rep_id'        => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:select;tag:SalesRepID;order:70'],
            'terms'         => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:Terms;order:80'],
            'notes'         => ['format'=>'VARCHAR(255)', 'attr'=>"DEFAULT NULL",           'comment'=>'tag:Notes;order:90'],
            'contact_id_b'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:BillingContactID;order:2'],
            'address_id_b'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:BillingAddressID;order:3'],
            'primary_name_b'=> ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingPrimaryName;order:10'],
            'contact_b'     => ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingAttention;order:20'],
            'address1_b'    => ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingAddress1;order:30'],
            'address2_b'    => ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingAddress2;order:40'],
            'city_b'        => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingCity;order:50'],
            'state_b'       => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingState;order:60'],
            'postal_code_b' => ['format'=>'VARCHAR(10)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingPostalCode;order:70'],
            'country_b'     => ['format'=>'CHAR(3)',      'attr'=>"DEFAULT NULL",           'comment'=>'type:country;tag:BillingCountryISO3;order:80'],
            'telephone1_b'  => ['format'=>'VARCHAR(20)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingTelephone1;order:90'],
            'email_b'       => ['format'=>'VARCHAR(64)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:BillingEmail;order:95'],
            'contact_id_s'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ShippingContactID;order:2'],
            'address_id_s'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ShippingAddressID;order:3'],
            'primary_name_s'=> ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingPrimaryName;order:10'],
            'contact_s'     => ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingAttention;order:20'],
            'address1_s'    => ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingAddress1;order:30'],
            'address2_s'    => ['format'=>'VARCHAR(36)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingAddress2;order:40'],
            'city_s'        => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingCity;order:50'],
            'state_s'       => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingState;order:60'],
            'postal_code_s' => ['format'=>'VARCHAR(24)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingPostalCode;order:70'],
            'country_s'     => ['format'=>'CHAR(3)',      'attr'=>"DEFAULT NULL",           'comment'=>'type:country;tag:ShippingCountryISO3;order:80'],
            'telephone1_s'  => ['format'=>'VARCHAR(20)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingTelephone1;order:90'],
            'email_s'       => ['format'=>'VARCHAR(64)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:ShippingEmail;order:95'],
            'tax_rate_id'   => ['format'=>'INT(11)',      'attr'=>"NOT NULL default '0'",   'comment'=>'type:select;tag:TaxRateID;order:10'],
            'gl_acct_id'    => ['format'=>'VARCHAR(15)',  'attr'=>"DEFAULT NULL",           'comment'=>'type:ledger;tag:GLAccount;order:10'],
            'drop_ship'     => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:checkbox;tag:DropShip;order:5'],
            'method_code'   => ['format'=>'VARCHAR(20)',  'attr'=>"DEFAULT NULL",           'comment'=>'type:hidden;tag:MethodID;order:2'],
            'currency'      => ['format'=>'CHAR(3)',      'attr'=>"NOT NULL DEFAULT '$iso'",'comment'=>'type:hidden;tag:CurrencyISO;order:80'],
            'currency_rate' => ['format'=>'DOUBLE',       'attr'=>"NOT NULL DEFAULT '1'",   'comment'=>'type:hidden;tag:CurrencyExchangeRate;order:81'],
            'so_po_ref_id'  => ['format'=>'INT(11)',      'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:SoPoRefID;order:5'],
            'closed_date'   => ['format'=>'DATE',         'attr'=>"DEFAULT NULL",           'comment'=>'type:hidden;format:date;tag:ClosedDate;order:6'],
            'recur_id'      => ['format'=>'INT(11)',      'attr'=>"DEFAULT NULL",           'comment'=>'type:hidden;tag:RecurID;order:7'],
            'admin_id'      => ['format'=>'INT(11)',      'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:UserID;order:8'],
            'printed'       => ['format'=>'INT(11)',      'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:Printed;order:8'],
            'attach'        => ['format'=>"ENUM('0','1')",'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:Attachment;order:9']],
        'keys' => 'PRIMARY KEY (id), KEY period (period), KEY journal_id (journal_id),
            KEY post_date (post_date), KEY closed (closed), KEY contact_id_b (contact_id_b), KEY contact_id_s (contact_id_s),
            KEY invoice_num (invoice_num), KEY so_po_ref_id (so_po_ref_id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'journal_periods' => ['module' => 'phreebooks',
        'fields'=> [
            'period'     => ['format'=>'INT(11)',  'attr'=>"DEFAULT '0'", 'comment'=>'tag:Period;order:1'],
            'fiscal_year'=> ['format'=>'INT(11)',  'attr'=>"DEFAULT '0'", 'comment'=>'tag:FiscalYear;order:10'],
            'start_date' => ['format'=>'DATE',     'attr'=>"DEFAULT NULL",'comment'=>'type:date;tag:StartDate;order:20'],
            'end_date'   => ['format'=>'DATE',     'attr'=>"DEFAULT NULL",'comment'=>'type:date;tag:EndDate;order:30'],
            'date_added' => ['format'=>'DATE',     'attr'=>"DEFAULT NULL",'comment'=>'type:date;tag:DateAdded;order:40'],
            'last_update'=> ['format'=>'TIMESTAMP','attr'=>"DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP",'comment'=>'type:date;tag:LastUpdate;order:50']],
        'keys' => 'PRIMARY KEY (period)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'phreeform' => ['module' => 'phreeform',
        'fields'=> [
            'id'         => ['format'=>'INT(10)',     'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'parent_id'  => ['format'=>'INT(10)',     'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:ParentID;order:2'],
            'group_id'   => ['format'=>'VARCHAR(10)', 'attr'=>"DEFAULT ''",             'comment'=>'type:hidden;tag:GroupID;order:3'],
            'mime_type'  => ['format'=>'VARCHAR(4)',  'attr'=>"DEFAULT NULL",           'comment'=>'type:hidden;tag:MimeType;order:4'],
            'security'   => ['format'=>'VARCHAR(255)','attr'=>"DEFAULT 'u:-1;g:-1'",    'comment'=>'type:hidden;tag:Security;order:5'],
            'title'      => ['format'=>'VARCHAR(64)', 'attr'=>"DEFAULT ''",             'comment'=>'tag:Title;order:10'],
            'bookmarks'  => ['format'=>'TEXT',        'attr'=>"DEFAULT NULL",           'comment'=>'type:checkbox;tag:Bookmarks;order:50'],
            'create_date'=> ['format'=>'DATE',        'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:CreateDate;order:70'],
            'last_update'=> ['format'=>'DATE',        'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:LastUpdate;order:80'],
            'doc_data'   => ['format'=>'TEXT',        'attr'=>"",                       'comment'=>'tag:DocData;order:90']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;'],
    'phreemsg' => ['module' => 'bizuno',
        'fields'=> [
            'id'       => ['format'=>'INT(10)',     'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'msg_id'   => ['format'=>'VARCHAR(32)', 'attr'=>"DEFAULT ''",             'comment'=>'type:hidden;tag:MessageID;order:2'],
            'status'   => ['format'=>'CHAR(1)',     'attr'=>"DEFAULT '0'",            'comment'=>'type:hidden;tag:Status;order:3'], // changed from 2 to 0 in R2.0.14
            'post_date'=> ['format'=>'DATETIME',    'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:PostDate;order:10'],
            'subject'  => ['format'=>'VARCHAR(255)','attr'=>"DEFAULT ''",             'comment'=>'tag:MessageSubject;order:20'],
            'body'     => ['format'=>'TEXT',        'attr'=>"",                       'comment'=>'tag:MessageBody;order:30']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;'],
    'roles' => ['module' => 'bizuno',
        'fields' => [
            'id'      => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'inactive'=> ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:checkbox;tag:Inactive;order:10'],
            'title'   => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:Title;order:20'],
            'settings'=> ['format'=>'TEXT',         'attr'=>"",                       'comment'=>'type:json;tag:Settings;order:30']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'tax_rates' => ['module' => 'phreebooks',
        'fields'=> [
            'id'        => ['format'=>'INT(3)',       'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'type'      => ['format'=>'VARCHAR(1)',   'attr'=>"NOT NULL DEFAULT 'c'",   'comment'=>'type:hidden;tag:Type;order:2'],
            'inactive'  => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:checkbox;tag:Inactive;order:20'],
            'title'     => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT Null",           'comment'=>'tag:Title;order:30'],
            'tax_rate'  => ['format'=>'FLOAT',        'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:TaxRate;order:40'],
            'start_date'=> ['format'=>'DATE',         'attr'=>"DEFAULT '2000-01-01'",   'comment'=>'type:date;tag:StartDate;order:50'],
            'end_date'  => ['format'=>'DATE',         'attr'=>"DEFAULT '2029-12-31'",   'comment'=>'type:date;tag:EndDate;order:60'],
            'settings'  => ['format'=>'TEXT',         'attr'=>"",                       'comment'=>'type:json;tag:Settings;order:70;format:stringify']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'tax_rates_table' => ['module' => 'phreebooks',
        'fields'=> [
            'ZipCode'              => ['format'=>'INT(1)',      'attr'=>"DEFAULT NULL",'comment'=>'tag:ZipCode;order:10'],
            'State'                => ['format'=>'VARCHAR(2)',  'attr'=>"DEFAULT NULL",'comment'=>'tag:State;order:20'],
            'TaxRegionName'        => ['format'=>"DECIMAL(7,6)",'attr'=>"DEFAULT NULL",'comment'=>'tag:TaxRegionName;order:30'],
            'StateRate'            => ['format'=>'DECIMAL(7,6)','attr'=>"DEFAULT NULL",'comment'=>'tag:StateRate;order:40'],
            'EstimatedCombinedRate'=> ['format'=>'DECIMAL(7,6)','attr'=>"DEFAULT NULL",'comment'=>'tag:EstimatedCombinedRate;order:50'],
            'EstimatedCountyRate'  => ['format'=>'DECIMAL(7,6)','attr'=>"DEFAULT NULL",'comment'=>'tag:EstimatedCountyRate;order:60'],
            'EstimatedCityRate'    => ['format'=>'DECIMAL(7,6)','attr'=>"DEFAULT NULL",'comment'=>'tag:EstimatedCityRate;order:70'],
            'EstimatedSpecialRate' => ['format'=>'DECIMAL(7,6)','attr'=>"DEFAULT NULL",'comment'=>'tag:EstimatedSpecialRate;order:80'],
            'RiskLevel'            => ['format'=>'INT(2)',      'attr'=>"DEFAULT NULL",'comment'=>'tag:RiskLevel;order:90']],
        'keys' => 'PRIMARY KEY (ZipCode)',
        'attr' => 'ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'users' => ['module' => 'bizuno',
        'fields' => [
            'admin_id'  => ['format'=>'INT(11)',      'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'email'     => ['format'=>'VARCHAR(96)',  'attr'=>"NOT NULL DEFAULT ''",    'comment'=>'tag:Email;order:10'],
            'title'     => ['format'=>'VARCHAR(32)',  'attr'=>"DEFAULT NULL",           'comment'=>'tag:Title;order:20'],
            'role_id'   => ['format'=>'INT(3)',       'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:select;tag:RoleID;order:30'],
            'contact_id'=> ['format'=>'INT(11)',      'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'tag:ContactID;order:40'],
            'inactive'  => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:checkbox;tag:Inactive;order:50'],
            'attach'    => ['format'=>"ENUM('0','1')",'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:Attachment;order:60'],
            'cache_date'=> ['format'=>'DATETIME',     'attr'=>"DEFAULT NULL",           'comment'=>'type:date;tag:CacheDate;order:70'],
            'settings'  => ['format'=>'TEXT',         'attr'=>"",                       'comment'=>'type:json;tag:Settings;order:80']],
        'keys' => 'PRIMARY KEY (admin_id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    'users_profiles' => ['module' => 'bizuno',
        'fields' => [
            'id'          => ['format'=>'INT(11)',    'attr'=>"NOT NULL AUTO_INCREMENT",'comment'=>'type:hidden;tag:RecordID;order:1'],
            'user_id'     => ['format'=>'INT(11)',    'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:UserID;order:2'],
            'menu_id'     => ['format'=>'VARCHAR(32)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'type:hidden;tag:MenuID;order:3'],
            'module_id'   => ['format'=>'VARCHAR(32)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'type:hidden;tag:ModuleID;order:4'],
            'dashboard_id'=> ['format'=>'VARCHAR(32)','attr'=>"NOT NULL DEFAULT ''",    'comment'=>'type:hidden;tag:DashboardID;order:5'],
            'column_id'   => ['format'=>'INT(3)',     'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:ColumnID;order:6'],
            'row_id'      => ['format'=>'INT(3)',     'attr'=>"NOT NULL DEFAULT '0'",   'comment'=>'type:hidden;tag:RowID;order:7'],
            'settings'    => ['format'=>'TEXT',       'attr'=>"",                       'comment'=>'type:json;tag:Settings;order:8']],
        'keys' => 'PRIMARY KEY (id)',
        'attr' => 'ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci'],
    ];
