<?php
/*
 * Payment Method - Payfabric (EVO Payments)
 *
 * NOTICE OF LICENSE
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/OSL-3.0
 *
 * DISCLAIMER
 * Do not edit or add to this file if you wish to upgrade Bizuno to newer
 * versions in the future. If you wish to customize Bizuno for your
 * needs please refer to http://www.phreesoft.com for more information.
 *
 * @name       Bizuno ERP
 * @author     Dave Premo, PhreeSoft <support@phreesoft.com>
 * @copyright  2008-2023, PhreeSoft, Inc.
 * @license    http://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * @version    6.x Last Update: 2023-06-01
 * @filesource /controllers/payment/methods/payfabric.php
 *
 * Source Information:
 * © 2022 PayFabric from EVO Payments
 * @link https://github.com/PayFabric - Main Documentation Site
 *
 * instructions on where/how to get account and fill out settings
 * setting for types of cards/payment to accept
 * setting to void or delete for same day journal deletions, returns for posted payments
 * setting to require AVS or no charge, notify/process anyway
 * settings for authorize only, sale, delete, void, return/credit, AVS (address verification)
 * accept credit cards, debit cards, EBT (Food Stamps), OPTIONAL, Gift Cards, electronic checks, PINless debit
 * OPTIONAL tip processing, EBT balance inquiry, Gift Card Balance inquiry, recurring payments, installments, attach signature
 */

namespace bizuno;

if (!defined('PAYMENT_PAYFABRIC_TOKEN'))     { define('PAYMENT_PAYFABRIC_TOKEN',     'https://www.payfabric.com/payment/api/token/create'); }
if (!defined('PAYMENT_PAYFABRIC_TOKEN_TEST')){ define('PAYMENT_PAYFABRIC_TOKEN_TEST','https://sandbox.payfabric.com/payment/api/token/create'); }
if (!defined('PAYMENT_PAYFABRIC_URL'))       { define('PAYMENT_PAYFABRIC_URL',       'https://www.payfabric.com/'); }
if (!defined('PAYMENT_PAYFABRIC_URL_TEST'))  { define('PAYMENT_PAYFABRIC_URL_TEST',  'https://sandbox.payfabric.com/'); }

class payfabric
{
    public  $moduleID = 'payment';
    public  $methodDir= 'methods';
    public  $code     = 'payfabric';
    private $mode     = 'prod'; // choices are 'test' (Test) or 'prod' (Production)

    public function __construct()
    {
        $this->lang    = getMethLang   ($this->moduleID, $this->methodDir, $this->code);
        $pmtDef        = getModuleCache($this->moduleID, 'settings', 'general', false, []);
        $this->settings= ['cash_gl_acct'=>$pmtDef['gl_payment_c'],'disc_gl_acct'=>$pmtDef['gl_discount_c'],'order'=>10,
            'setup_id'=>'','device_id'=>'','device_pw'=>'','prefix'=>'CC','allowRefund'=>'0']; // ,'prefixAX'=>'AX'
        $usrSettings   = getModuleCache($this->moduleID, $this->methodDir, $this->code, 'settings', []);
        settingsReplace($this->settings, $usrSettings, $this->settingsStructure());
    }

    public function settingsStructure()
    {
        return [
            'cash_gl_acct'=> ['label'=>$this->lang['gl_payment_c_lbl'], 'position'=>'after','attr'=>['type'=>'ledger','id'=>"{$this->code}_cash_gl_acct",'value'=>$this->settings['cash_gl_acct']]],
            'disc_gl_acct'=> ['label'=>$this->lang['gl_discount_c_lbl'],'position'=>'after','attr'=>['type'=>'ledger','id'=>"{$this->code}_disc_gl_acct",'value'=>$this->settings['disc_gl_acct']]],
            'order'       => ['label'=>lang('order'),             'position'=>'after','attr'=>['type'=>'integer', 'size'=>'3','value'=>$this->settings['order']]],
            'setup_id'    => ['label'=>$this->lang['setup_id'],   'position'=>'after','attr'=>['size'=>'32','value'=>$this->settings['setup_id']]],
            'device_id'   => ['label'=>$this->lang['device_id'],  'position'=>'after','attr'=>['size'=>'48','value'=>$this->settings['device_id']]],
            'device_pw'   => ['label'=>$this->lang['device_pw'],  'position'=>'after','attr'=>['size'=>'24','value'=>$this->settings['device_pw']]],
            'prefix'      => ['label'=>$this->lang['prefix_lbl'], 'position'=>'after','attr'=>['size'=>'5','value'=>$this->settings['prefix']]],
//          'prefixAX'    => ['label'=>$this->lang['prefix_amex'],'position'=>'after','attr'=>['size'=>'5','value'=>$this->settings['prefixAX']]],
            'allowRefund' => ['label'=>$this->lang['allow_refund'],                   'attr'=>['type'=>'selNoYes','value'=>$this->settings['allowRefund']]]];
    }

    public function render($layout, $values=[], $dispFirst=false)
    {
        msgDebug("\nEntering $this->code render, working with values = ".print_r($values, true));
        $this->cID = clean($layout['fields']['contact_id_b']['attr']['value'], 'integer');
        $fields= [
            'payment_id'=> ['attr'=>['type'=>'hidden']],
            'trans_code'=> ['attr'=>['type'=>'hidden']],
            'selWallet' => ['values'=>[],'attr'=>['type'=>'select'],'events'=>['onChange'=>"{$this->code}RefNum('stored');"]],
            'refresh'   => ['attr'=>['type'=>'button','value'=>'Reload'],      'events'=>['onClick'=>"jsonAction('payment/wallet/reload', $this->cID);"]],
            'addCard'   => ['attr'=>['type'=>'button','value'=>'Add Card'],    'events'=>['onClick'=>"jsonAction('payment/wallet/add',$this->cID);"]],
//          'capCard'   => ['attr'=>['type'=>'button','value'=>'Capture Sale'],'events'=>['onClick'=>"jsonAction('payment/main/sale', $this->cID);"]],
            ];
        if (!empty($values['method']) && $values['method']==$this->code && !empty($layout['fields']['id']['attr']['value'])) { // edit
            $isEdit = true;  // since it's an edit, all adjustments need to be made at the gateway, this prevents duplicate charges when re-posting a transaction
            $invoice_num = $invoice_amex = $layout['fields']['invoice_num']['attr']['value'];
            $gl_account  = $layout['fields']['gl_acct_id']['attr']['value'];
            $discount_gl = $this->getDiscGL($layout['fields']['id']['attr']['value']);
        } else { // defaults
            $isEdit = false;
            $invoice_num = $this->settings['prefix'].biz_date('Ymd');
            $gl_account  = $this->settings['cash_gl_acct'];
            $discount_gl = $this->settings['disc_gl_acct'];
            if (!empty($this->cID)) { // find if stored values
                $pfID  = 'C'.str_pad($this->cID, 9, '0', STR_PAD_LEFT);
                $temp = $this->walletList($pfID);
                $cards = [];
                foreach ($temp as $card) { $cards[] = ['id'=>$card['id'],'text'=>$card['text']]; }// trim the list a bit
                $fields['selWallet']['values'] = $cards;
            }
            if (!empty($values['trans_code'])) {
                $invoice_num = isset($values['hint']) && substr($values['hint'], 0, 2)=='37' ? $invoice_amex : $invoice_num;
                $fields['trans_code']['attr']['value'] = $values['trans_code'];
            }
        }
        htmlQueue("arrPmtMethod['$this->code'] = {cashGL:'$gl_account', discGL:'$discount_gl', ref:'$invoice_num'};\n".$this->eventJS($this->cID), 'jsHead');
        if ($this->code == $dispFirst) { htmlQueue("bizTextSet('invoice_num', '$invoice_num');", 'jsReady'); }
        $html  =
html5($this->code.'_action', ['label'=>$this->lang['stored'],      'attr'=>['type'=>'radio','value'=>'s','checked'=>!$isEdit?true:false],'hidden'=>$isEdit?true:false,
    'events'=>  ['onChange'=>"jqBiz('#div{$this->code}s').show();"]]).
html5($this->code.'_action', ['label'=>$this->lang['at_payfabric'],'attr'=>['type'=>'radio','value'=>'w','checked'=> $isEdit?true:false],
    'events'=>  ['onChange'=>"jqBiz('#div{$this->code}s').hide();"]]).'<br />';
        $html .= '<div id="div'.$this->code.'s"'.($isEdit?' style=" display:none"':'').'>';
        $html .= lang('payment_stored_cards').'<br />'.html5($this->code.'selWallet', $fields['selWallet']);
        $html .= '<p>'.html5($this->code.'addCard',  $fields['refresh']).'&nbsp;&nbsp;&nbsp;';
        $html .= html5($this->code.'addCard',  $fields['addCard']).'</p>';
//      $html .= '<p>'.html5($this->code.'capCard',  $fields['capCard']).'</p>';  // don't add this, use the Save button so all the fields are posted
        $html .= '</div>';

//        $html  = '';
//        $html .= '<div id="div'.$this->code.'">';
//        $html .= '<p>'.html5($this->code.'selWallet',$fields['selWallet']).'</p>';
//        $html .= '<p>'.html5($this->code.'addCard',  $fields['addCard']).'</p>';
//        $html .= '<p>'.html5($this->code.'capCard',  $fields['capCard']).'</p>';  // don't add this, use the Save button so all the fields are posted
//        $html .= '</div>';
        return $html;
    }

    public function eventJS($cID)
    {
        $subDom = ($this->mode=='test' ? 'sandbox' : 'www');
        return "
function payment_$this->code() {
    bizTextSet('invoice_num', arrPmtMethod['$this->code'].ref);
    bizSelSet('gl_acct_id', arrPmtMethod['$this->code'].cashGL);
    bizSelSet('totals_discount_gl', arrPmtMethod['$this->code'].discGL);
}
function {$this->code}RefNum(type) {
    if (type=='stored') { var ccNum = jqBiz('#{$this->code}selWallet').val(); }
      else { var ccNum = jqBiz('#{$this->code}_number').val();  }
    bizTextSet('invoice_num', arrPmtMethod['$this->code'].ref);
}
function payFabricSale() { payFabricCardID = bizSelGet('{$this->code}selWallet'); }
function {$this->code}WalletEvent(event) {
    var objMsg;
    objMsg = JSON.parse(event.data);
    if (event.origin !== 'https://$subDom.payfabric.com') { return; }
    switch(objMsg.Event) {
        case 'btn_Close':                  jqBiz('#pmtIFrame').remove();        break;
        case 'OnSaveTransactionCompleted': alert('OnSaveTransactionCompleted'); break;
        case 'OnTransactionCompleted':     alert('OnTransactionCompleted');     break;
        case 'OnWalletCreateCompleted':
        case 'OnWalletUpdateCompleted':
            jqBiz('#pmtIFrame').remove();
            window.removeEventListener('message', {$this->code}WalletEvent, false);
            bizWindowClose('winIFrame');
            if (jqBiz('#wallet').length)                { bizPanelRefresh('wallet'); }
            if (jqBiz('#{$this->code}selWallet').length){ jsonAction('payment/wallet/reload', $cID); }
            break;
    }
}
window.addEventListener('message', {$this->code}WalletEvent, false);";
    }

/* methods to write
** Version 3.0 **
* Payment Gateway Profiles
- Retrieve a Payment Gateway Profile
- Retrieve Payment Gateway Profiles
- Manual Batch Close
* Addresses
- Retrieve a Shipping Address
- Retrieve Shipping Addresses
* JSON Web Tokens
- Create JWT Token
- Validate JWT Token
** Version 3.1 **
* Email Transaction Receipt
- Send a Transaction Receipt
* Email Transaction Receipt Template
- Get Email Receipt Templates
- Update Email Receipt Template
* Batched Transactions
- Get Current Batches
- Process Batch
- Delete Batch
- Search by Batch Number
* Scheduled Transactions
- Search for Future Dated Transactions
* Custom Reports
- Get Custom Reports
- Create Custom Report
- Edit Custom Report
- Delete Custom Report
- Manual Execute Custom Report
* Shipping Addresses
- Create Shipping Address
- Delete Shipping Address
* Payment Terminal
- Get Registered Terminals
- Create new Registered Terminal
- Update Registered Terminal
- Remove Registered Terminal
- Get Terminal Settings
- Update Terminal Settings
* Products
- Get Products
- Create Product
- Edit Product
- Delete Product
- Upload Products
* Theme
- Get Themes
- Create Theme
- Update Theme
- Delete Theme
* Transaction Adjustments
- Transaction Adjustments
*/
    // ***************************************************************************************************************
    //                                Transaction Methods
    // ***************************************************************************************************************
    /**
     * @param type $fields
     * @param type $ledger
     * @return type
     */
    public function authorization()
    {
        msgAdd('PhreeSoft is working on this feature, please process credit cards directly on the PayFabric website.');
        // The user has selected a card from the list or entered a new card, at this point we have the cardID
//        $cardID = clean('cardID', 'alpha_num', 'get');
        // do the transaction to just authorize the card
        // if failed, return false, else return array
//        return ['txID'=>$resp->ssl_txn_id, 'txTime'=>$resp->ssl_txn_time, 'code'=>$resp->ssl_approval_code];
    }

    /**
     * 'process' - Captures a pre-authorized sale, requires transaction ID for authorization
     * @param type $fields
     * @param type $ledger
     */
    public function capture()
    {
        msgAdd('PhreeSoft is working on this feature, please process credit cards directly on the PayFabric website.');
        // The user has selected a card from the list or entered a new card, at this point we have the cardID
//        $cardID = clean('cardID', 'alpha_num', 'get');
        // do the transaction to capture the card
        // if failed, return false, else return array
//        return ['txID'=>$resp->ssl_txn_id, 'txTime'=>$resp->ssl_txn_time, 'code'=>$resp->ssl_approval_code];
    }

    /**
     * 'ship' - This method will capture payment, if payment was authorized in a prior transaction, a ccComplete is done
     * @param integer $fields - record id from table journal_main to generate the capture, the transaction ID will be pulled from there.
     * @return array - On success, false (with messageStack message) on unsuccessful deletion
     */
    public function sale($fields, $ledger)
    {
//      msgDebug("\nEntering $this->code paymentSale working with fields = ".print_r($fields, true));
        switch ($fields['action']) {
            case 'w': // website capture, just post it
                msgAdd($this->lang['msg_capture_manual'].' '.$this->lang['msg_website'], 'caution');
                return true;
            default: // default to get the card ID and process
        }
        $custID = 'C'.str_pad($ledger->main['contact_id_b'], 9, '0', STR_PAD_LEFT);
        $cardID = clean('payfabricselWallet', 'cmd', 'post');
        if (empty($cardID)) { return msgAdd("No card information was provided!"); }
        if (!$this->walletValidate($custID, $cardID)) {
            return msgAdd("Error - the card ID provived ins't in the customers wallet! Please submit a support ticket right away describing the steps taken to get this error. PhreeSoft needs to get this bug fixed ASAP.", 'trap');
        }
        $this->mains = [];
        foreach ($ledger->items as $item) { // load the invoices
            if ($item['gl_type'] <> 'pmt') { continue; }
            $this->mains[] = dbGetRow(BIZUNO_DB_PREFIX.'journal_main', "invoice_num='{$item['trans_code']}' AND journal_id IN (12, 13)");
        }
        $salesTax= $this->calculateTax();
        $request = [
            'Type'    => 'Sale',
            'SetupId' => $this->settings['setup_id'], // 'EVO US_CC', // specific to PhreeSoft Test for CC processing
            'Amount'  => number_format($ledger->main['total_amount'], 2, '.', ''),
            'Card'    => ['ID'=> $cardID],
            'Currency'=> 'USD',
            'Customer'=> $custID,
            'Tender'  => 'CreditCard',
//          'BatchNumber' => '', 'PayDate' => '', 'ReferenceKey' => null, 'ReferenceTrxs' => [], 'ReqAuthCode' => '',
//          'Shipto' => ['City'=>'','Country'=>'','Customer'=>'','Email'=>'','Line1'=>'','Line2'=>'','Line3'=>'','Phone'=>'','State'=>'','Zip'=>''],
//          'TrxUserDefine1' => '', 'TrxUserDefine2' => '', 'TrxUserDefine3' => '', 'TrxUserDefine4' => '',
            'Document'=> [
                'Head'=> [
                    [ 'Name'=> 'InvoiceNumber', 'Value'=> $this->calculateInvoice() ], // cash receipt number, something to track back
                    [ 'Name'=> 'PONumber',      'Value'=> $this->calculatePONumber() ],
                    [ 'Name'=> 'DiscountAmount','Value'=> $ledger->main['discount'] ],
                    [ 'Name'=> 'FreightAmount', 'Value'=> $this->calculateShipping() ],
                    [ 'Name'=> 'HandlingAmount','Value'=> '0.00' ],
                    [ 'Name'=> 'TaxExempt',     'Value'=> empty($salesTax) ? 'Y' : 'N' ],
                    [ 'Name'=> 'TaxAmount',     'Value'=> $salesTax ],
//                  [ 'Name'=> 'DutyAmount',    'Value'=> '0.00' ],
//                  [ 'Name'=> 'VATTaxAmount',  'Value'=> '' ],
//                  [ 'Name'=> 'VATTaxRate',    'Value'=> '' ],
//                  [ 'Name'=> 'ShipFromZip',   'Value'=> '' ],
//                  [ 'Name'=> 'ShipToZip',     'Value'=> '' ],
                    [ 'Name'=> 'OrderDate',     'Value'=> date('m/d/Y', strtotime($ledger->main['post_date'])) ]],
                'UserDefined'=>[
                    [ 'Name'=> 'AppID',         'Value'=>'PhreeSoft_Bizuno']],
                ]];
/*      foreach ($ledger->items as $item) { // Since this can be a bulk payment, we don't know the items. skip for now
            $request['Document']['Lines'][] = [
                'Columns'=> [
                    [ 'Name'=> 'ItemCommodityCode', 'Value'=> '24X BIKE' ],
                    [ 'Name'=> 'ItemProdCode', 'Value'=> 'B872' ],
                    [ 'Name'=> 'ItemUPC', 'Value'=> 'B872' ],
                    [ 'Name'=> 'ItemUOM', 'Value'=> 'SET' ],
                    [ 'Name'=> 'ItemDesc', 'Value'=> 'Mountain Bicycle' ],
                    [ 'Name'=> 'ItemAmount', 'Value'=> '2000.00' ],
                    [ 'Name'=> 'ItemCost', 'Value'=> '112.00' ],
                    [ 'Name'=> 'ItemDiscount', 'Value'=> '100.00' ],
                    [ 'Name'=> 'ItemFreightAmount', 'Value'=> '12.00' ],
                    [ 'Name'=> 'ItemHandlingAmount', 'Value'=> '10.00' ],
                    [ 'Name'=> 'ItemQuantity', 'Value'=> '10' ]]];
        }
*/
        msgDebug("\nReady to send sale request: ".print_r($request, true));
        $transURL= "payment/api/transaction/create"; // Optional: cvc={CVCValue}
        $tOptions= [CURLOPT_POSTFIELDS=>json_encode($request)]; // removed CURLOPT_VERBOSE=>true,
        $trans   = $this->queryAPI($transURL, $tOptions); // Create the transaction
        if (empty($trans)) { return; }
        $saleUrl = "payment/api/transaction/process/{$trans['Key']}";
        $sale    = $this->queryAPI($saleUrl); // Process the order
        if (empty($sale)) { return; }
        // Process the results
/*  [AVSAddressResponse] =>
    [AVSZipResponse] =>
    [AuthCode] => 34XTZW
    [CVV2Response] => NotSet
    [CardType] => Debit
    [ExpectedSettledTime] => 2022-05-24T03:00:00.0000000Z
    [FinalAmount] => 6.00
    [IAVSAddressResponse] =>
    [Message] => APPROVED
    [OrigTrxAmount] => 6.00
    [OriginationID] => E2666BDA8D834865BB23132D988CFCA7
    [PayFabricErrorCode] =>
    [RespTrxTag] => 5/23/2022 9:03:03 PM
    [ResultCode] => 1
    [SettledTime] =>
    [Status] => Approved
    [SurchargeAmount] => 0.00
    [SurchargePercentage] => 0.0
    [TerminalID] =>
    [TerminalResultCode] =>
    [TrxAmount] => 6.00
    [TrxDate] => 5/23/2022 3:03:03 PM
    [TrxDateUTC] => 5/23/2022 10:03:03 PM
    [TrxKey] => 22052301848600
    [WalletID] => b2624872-6456-4a4b-8778-adfb847a3877
*/
        if ($sale['Status']=='Approved') {
            if (!empty($sale['CVV2Response']) && $sale['CVV2Response'] != 'NotSet') {
                msgAdd(sprintf($this->lang['err_cvv_mismatch'], $this->lang["CVV_{$sale['CVV2Response']}"]));
            }
            if (!empty($sale['AVSZipResponse']) && !in_array($sale['AVSZipResponse'], ['X','Y'])) {
                msgAdd(sprintf($this->lang['err_avs_mismatch'], $this->lang["AVS_{$sale['AVSZipResponse']}"]));
            }
            $cvv = !empty($sale['CVV2Response']) ? $this->lang["CVV_{$sale['CVV2Response']}"] : 'n/a';
            msgAdd(sprintf($this->lang['msg_approval_success'], $sale['Message'], $sale['AuthCode'], $cvv), 'success');
            return ['txID'=>$trans['Key'], 'txTime'=>$sale['TrxDate'], 'code'=>$sale['AuthCode']];
        }
        // else $sale['Status'] => Failure
        msgLog(sprintf($this->lang['err_process_decline'], $sale['PayFabricErrorCode'], $sale['Message']));
        msgAdd(sprintf($this->lang['err_process_decline'], $sale['PayFabricErrorCode'], $sale['Message']));
    }

    /**
     * @method void will delete/void a payment made BEFORE the processor commits the payment, typically must be run the same day as the sale
     * @param integer $rID Record id from table journal_main to generate the void
     * @return array merchant response On success, false (with messageStack message) on unsuccessful deletion
     */
    public function void($rID=0)
    {
        if (!$rID) { return msgAdd('Bad record ID passed'); }
        $txID = dbGetValue(BIZUNO_DB_PREFIX."journal_item", 'trans_code', "ref_id=$rID AND gl_type='ttl'");
        if (!$txID || !$this->settings['allowRefund']) { msgAdd(lang('err_cc_no_transaction_id'), 'caution'); return true; }
        // create the transaction
        $voidUrl = "payment/api/reference/$txID?trxtype=Void";
        $void    = $this->queryAPI($voidUrl);
        if (empty($void)) { return; }
        msgLog(sprintf($this->lang['msg_void_success'], $void['Message'], $void['AuthCode']));
        msgAdd(sprintf($this->lang['msg_void_success'], $void['Message'], $void['AuthCode']), 'success');
        return ['txID'=>$txID, 'txTime'=>$void['TrxDate'], 'code'=>$void['AuthCode']];
    }

    /**
     * This method will refund a payment made AFTER the batch is processed, typically must be run any day after the sale
     * @param integer $rID - record id from table journal_main to generate the refund
     * @param float $amount - amount to be refunded (leave blank for full amount)
     * @return array - On success, false (with messageStack message) on unsuccessful deletion
     */
    public function refund($rID=0, $amount=false)
    {
        // @TODO - This may be an option to be performed during the credit memo refund in banking to save the user the hassle of refunding through the portal.
return msgAdd("Deleting a Payment after the batch has been processed has been disabled. To refund a customer after the batch has ben processed requires a Credit Memo and then payment applied against it. Then, the credit card refund needs to be completed throught the portal.");
        if (!$rID) { return msgAdd('Bad record ID passed'); }
        $results   = dbGetValue(BIZUNO_DB_PREFIX."journal_item", ['debit_amount', 'credit_amount', 'trans_code'], "ref_id=$rID AND gl_type='ttl'");
        $max_amount= $results['debit_amount'] + $results['credit_amount'];
        if ($amount === false) { $amount = $max_amount; }
        if ($amount > $max_amount)  { return msgAdd(lang('err_cc_amount_too_big')); }
        if (floatval($amount) <= 0) { return msgAdd(lang('err_cc_amount_negative')); }
        if (!$results['trans_code'] || !$this->settings['allowRefund']) { msgAdd(lang('err_cc_no_transaction_id'), 'caution'); return true; }
        // Full refund
        if ($amount <> $max_amount) {
            $voidUrl = "payment/api/reference/{$results['trans_code']}?trxtype=Refund";
            $void    = $this->queryAPI($voidUrl);
            if (empty($void)) { return; }
        } else {
            $transaction = [
                'Amount'      => number_format($amount, 2, '.', ''),
                'ReferenceKey'=> $results['trans_code'],
                'Type'        => 'Refund'];
            // Partial refunds
            $transURL= "payment/api/transaction/process";
            $options = [CURLOPT_POSTFIELDS=>json_encode($transaction)]; // removed CURLOPT_VERBOSE=>true,
            $refund  = $this->queryAPI($transURL, $options);
            if (empty($refund)) {
                msgAdd("The refund failed at PayFabric with a transaction code: {$refund['ResultCode']}. However, the cash receipt was still deleted from Bizuno. Please check at the PayFabric Gateway to make sure the charge was properly refunded.", 'caution');
                return true;
            }
        }
        msgLog(sprintf($this->lang['msg_refund_success'], $refund['Message'], $refund['AuthCode']));
        msgAdd(sprintf($this->lang['msg_refund_success'], $refund['Message'], $refund['AuthCode']), 'success');
        return ['txID'=>$refund['TrxKey'], 'txTime'=>$refund['TrxDate'], 'code'=>$refund['AuthCode']];
    }

    // ***************************************************************************************************************
    //                                Wallet Methods
    // ***************************************************************************************************************
    /**
     * Retrieves a list of cards and e-checks from the PayFabric wallet
     * @param type $pfID
     */
    public function walletEdit($pfID)
    {
        $httpUrl = "payment/api/wallet/get/$pfID?tender=CreditCard";
        $response= $this->queryAPI($httpUrl);
        $cards   = [];
        if (empty($response)) { return $cards; }
        foreach ($response as $card) {
            $hint = substr($card['Account'], -5);
            $cards[] = [
                'id'  => $card['Billto']['ID'],
                'text'=> "{$card['CardName']} ({$card['CardType']} - $hint)",
                'name'=> $card['CardName'],
                'type'=> $card['CardType'],
                'hint'=> $hint];
        }
        return $cards; // array of e-checks and cards in wallet.
    }

    public function walletDelete($cardID='')
    {
        $httpUrl = "payment/api/wallet/delete/$cardID";
        $response= $this->queryAPI($httpUrl);
        return !empty($response['Result']) && strtolower($response['Result'])=='true' ? true : false;
    }

    /**
     * Retrieves a list of cards and e-checks from the PayFabric wallet
     * @param type $pfID
     */
    public function walletList($pfID)
    {
        $httpUrl = "payment/api/wallet/get/$pfID?tender=CreditCard";
        $response= $this->queryAPI($httpUrl);
        $cards   = [];
        if (empty($response)) {
            msgDebug("\nReturning with no cards found!");
            return $cards;
        }
        foreach ($response as $row) {
            $hint = substr($row['Account'], -5);
            $cards[] = array_merge(['id'=>$row['ID'],'text'=>"{$row['CardName']} ({$row['CardType']} - $hint)",'hint'=>$hint], $row);
        }
        msgDebug("\nReturning from walletList with cards = ".print_r($cards, true));
        return sortOrder($cards, 'IsDefaultCard', 'desc'); // array of e-checks and cards in wallet with default first
    }

    public function walletReload(&$layout=[], $pfID=0)
    {
        $output = [];
        $cards = $this->walletList($pfID);
        foreach ($cards as $card) { $output[] = ['id'=>$card['id'], 'text'=>$card['text']]; }
        $action = "sel_{$this->code}selWallet = ".json_encode($output)."; bizSelReload('{$this->code}selWallet', sel_{$this->code}selWallet);";
        $layout  = array_replace_recursive($layout, ['content'=>['action'=>'eval','actionData'=>$action]]);
    }

    /**
     * Updates properties for a given customers wallet
     * @param string $pfID - Wallet ID
     * @param type $props - Fields within the wallet to update/change, use ['NewCustomerNumber'=>'CustID_TBD'] to change the CustomerID field
     * @return bool - true on success, false on error
     */
    public function walletRename($pfID, $props=[]) {
        $cards = $this->walletList($pfID); // Get all the cards from a specific wallet
        foreach ($cards as $card) {
            $opts    = array_replace($props, ['ID'=>$card['id']]);
            msgDebug("\nReady to send update request via POST: ".print_r($opts, true));
            $tOptions= [CURLOPT_POSTFIELDS=>json_encode($opts)];
            $resp    = $this->queryAPI('payment/api/wallet/update', $tOptions);
        }
        return !empty($resp['Result']) && strtolower($resp['Result'])=='true' ? true : false;
    }

    /**
     * Validates a cardID is within a customers wallet
     * @param string $custID - Customer ID @PayFabric
     * @param string $cardID - Card ID @PayFabric
     */
    public function walletValidate($custID=0, $cardID='')
    {
        $cards = $this->walletList($custID);
        foreach ($cards as $card) {
            if ($card['id']==$cardID) { return true; }
        }
        return false;
    }

    // ***************************************************************************************************************
    //                                Support Methods
    // ***************************************************************************************************************
    private function queryAPI($url, $opts=[])
    {
        $destURL = ($this->mode=='test' ? PAYMENT_PAYFABRIC_URL_TEST : PAYMENT_PAYFABRIC_URL).$url;
        $header  = ["Content-Type: application/json", "authorization: ".$this->settings['device_id']."|".$this->settings['device_pw']];
        $options = array_replace([CURLOPT_RETURNTRANSFER=>true, CURLOPT_HTTPHEADER=>$header], $opts);
        msgDebug("\nEntering PayFabric queryAPI with url: $destURL with options: ".print_r($options, true));
        $handle   = curl_init($destURL);
        curl_setopt_array($handle, $options);
        $respBody = curl_exec($handle);
        $respCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
        curl_close($handle);
        msgDebug("\nResponse Body from PayFabric: ".print_r($respBody, true));
        if ($respCode >= 300) { return msgAdd("PayFabric Transaction Error. Response code: $respCode"); }
        $response = json_decode($respBody, true);
        msgDebug("\ncURL response after decode = ".print_r($response, true));
        return $response;
    }

    public function getToken()
    {
        $httpUrl    = $this->mode=='test' ? PAYMENT_PAYFABRIC_TOKEN_TEST : PAYMENT_PAYFABRIC_TOKEN;
        $header = ["Content-Type: application/json", "authorization: ".$this->settings['device_id']."|".$this->settings['device_pw']];
        $curlOptions= [CURLOPT_RETURNTRANSFER=>true, CURLOPT_HTTPHEADER=>$header];
        msgDebug("\nPayFabric url: $httpUrl with header: ".print_r($header, true));
        $curlHandle = curl_init($httpUrl);
        curl_setopt_array($curlHandle, $curlOptions);
        $httpResponseBody = curl_exec($curlHandle);
        $httpResponseCode = curl_getinfo($curlHandle, CURLINFO_HTTP_CODE);
        curl_close($curlHandle);
        msgDebug("\nResponse Body from PayFabric: ".print_r($httpResponseBody, true));
        if ($httpResponseCode >= 300) { return msgAdd("PayFabric Token request Error. Response code: $httpResponseCode"); }
        $response = json_decode($httpResponseBody, TRUE);
        $this->settings['token']     = $response['Token'];
//      $this->settings['token_date']= time()+$response['expires_in']; // no expiration time provided so get a new token everytime
        $methProps = getModuleCache($this->moduleID, $this->methodDir, $this->code);
        $methProps['settings']       = $this->settings;
        setModuleCache($this->moduleID, $this->methodDir, $this->code, $methProps);
        return $response['Token'];
    }
    private function calculateInvoice()
    {
        $output = [];
        foreach ($this->mains as $main) { $output[] = !empty($main['invoice_num']) ? $main['invoice_num'] : 'N/A'; }
        return implode(',', $output);
    }
    private function calculatePONumber()
    {
        $output = [];
        foreach ($this->mains as $main) { $output[] = !empty($main['purch_order_id']) ? $main['purch_order_id'] : 'N/A'; }
        return implode(',', $output);
    }
    private function calculateShipping()
    {
        $total = 0;
        foreach ($this->mains as $main) { $total += $main['freight']; }
        return number_format($total, 2, '.', '');
    }
    private function calculateTax()
    {
        $total = 0;
        foreach ($this->mains as $main) { $total += $main['sales_tax']; }
        return number_format($total, 2, '.', '');
    }
    /**
     *
     * @param type $data
     * @return type
     */
    private function getDiscGL($data)
    {
        if (isset($data['fields'])) {
            foreach ($data['fields'] as $row) {
                if ($row['gl_type'] == 'dsc') { return $row['gl_account']; }
            }
        }
        return $this->settings['disc_gl_acct']; // not found, return default
    }

    /**
     * Tries to guess the invoice number and po number of the first pmt record of the item array
     * @param type $ledger
     * @return type
     */
    private function guessInv($ledger)
    {
        $refs = ['inv'=>$ledger->main['invoice_num'], 'po'=>$ledger->main['invoice_num']];
        if (empty($ledger->items)) { return $refs; }
        foreach ($ledger->items as $row) {
            if ($row['gl_type'] <> 'pmt') { continue; } // just the first row
            $vals = explode(' ', $row['description'], 4);
            if (!empty($vals[1])) { $refs['inv']= $vals[1]; }
            if (!empty($vals[3])) { $refs['po'] = $vals[3]; }
            break;
        }
        return $refs;
    }

    private function setAddress($data)
    {
        "ID	Guid	Unique identifier for this record	36
        Customer	String	ID for this customer. This is generated by the client upon creation of the customer.	nvarchar(128)
        Country	String	Country name	varchar(64)
        State	String	State name	varchar(64)
        City	String	City name	varchar(64)
        Line1	String	Street line 1	nvarchar(128)
        Line2	String	Street line 2	nvarchar(128)
        Line3	String	Street line 3	nvarchar(128)
        Email	String	Email address	nvarchar(128)
        Phone	String	Phone number	varchar(16)
        ModifiedOn	String	Timestamp indicating when this record was last modified. It's format should like \"3/23/2015 11:16:19 PM\".	datetime,not null
        Zip	String	Zip code	varchar(16)";

    }

    private function setCard($data)
    {
        'ID	Guid	Unique identifier for a card or wallet record. This ID (aka Wallet ID) is generated by PayFabric upon successful creation of a new card entry for either credit card or ACH. The client cannot set or modify this value.
        Tender*	Options	Tender type. Valid options are CreditCard, ECheck, ApplePay and GooglePay.	nvarchar(64)
        Customer*	String	Customer ID or Customer Number as specified by the client upon creation of the customer. This is a required field in order to associate a card/wallet entry to a customer account.	nvarchar(128)
        Account*	String	The number of the credit card, or the eCheck/ACH account. When creating a new Card this attribute must be provided by the client in plaintext. When a client retrieves a card PayFabric always returns this attribute in masked format. Ignore this attribute when update a existing card.	nvarchar(64)
        ExpDate*	String	Expiration date of the credit card in the format of MMYY. Only valid for credit cards.	varchar(4)
        CheckNumber	String	Check number. Only valid for eChecks, and required for specific Processors (TeleCheck, Encompass).	varchar(128)
        AccountType	String	eCheck account type. Only valid for eCheck accounts.	varchar(32)
        Aba*	String	Bank Routing Number. Only valid for eChecks.	varchar(64)
        CardName	String	Type of credit card: Visa, Mastercard, Discover,JCB,AmericanExpress,DinersClub. Only valid for credit cards.	nvarchar(16)
        IsDefaultCard	Boolean	Indicates whether this is the primary card of the customer. Default value is False.	bit, not null
        IsLocked	Boolean	Indicates whether the card is locked. Default value is False.	bit, null
        IsSaveCard	Boolean	Indicates whether to save this card in the customer\'s wallet. This attribute is only valid and should only be included in the object when using Create and Process a Transaction. And it will be set to false automatically for Verify transactions or transactions with Tender set to ApplePay or GooglePay.
        ModifiedOn	String	This is a response field. Timestamp indicating when this record was last modified. It\'s format should like "3/23/2015 11:16:19 PM".	datetime, not null
        CardHolder*	Object	Cardholder object.
        Billto	Object	Address object.
        Identifier	String	A client-defined identifier for this card. Developer can send a flag value to identify this card	nvarchar(32)
        UserDefine1	String	User-defined field 1. Developer can store additional data in this field.	nvarchar(256)
        UserDefine2	String	User-defined field 2. Developer can store additional data in this field.	nvarchar(256)
        UserDefine3	String	User-defined field 3. Developer can store additional data in this field.	nvarchar(256)
        UserDefine4	String	User-defined field 4. Developer can store additional data in this field.	nvarchar(256)
        Connector	String	The gateway name defined by PayFabric such as FirstDataGGe4, PayflowPro or Paymentech. This field will be set only if this card is a tokenized value for a specific gateway, such as FirstData or Paypal	nvarchar(64)
        GatewayToken	String	Gateway token. PayFabric send this value to gateway for processing a transaction	varchar(32)
        IssueNumber	String	This field is required for UK debit cards	nvarchar(64)
        StartDate	String	This field is required for UK debit cards, format is MMYY.	varchar(4)
        NewCustomerNumber	String	This field is used to submit new customer number for updating this record\'s customer field.	nvarchar(128)
        CardType	String	This is a response field, the possible value is \'Credit\', \'Debit\' or \'Prepaid\' for credit card, and it is blank for eCheck.	varchar(20)
        EncryptedToken	String	The Apple Pay or Google Pay payment token, as provided by the provider. Conditionally required if Tender is ApplePay or GooglePay.	nvarchar(MAX)';
    }

    private function setCardHolder($data)
    {
        "DriverLicense	String	Driver license	varchar(32)
        FirstName*	String	First name	nvarchar(64)
        LastName*	String	Last name	nvarchar(64)
        MiddleName	String	Middle name	nvarchar(64)
        SSN	String	Social security number	varchar(16)";
    }

}
